/*
 * Created on 09.03.2004
 * MultipartMailClient
 *
*/
package theorie;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Properties;

import javax.mail.Folder;
import javax.mail.Header;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.URLName;

/**
 * Multipart Message mit mehreren angehngten Dateien
 * Multipart MIME
 */

public class MultipartMailClient {
	private static final String URL = "imap://jjoller@imap.hsr.ch/INBOX";
	public static void main(String[] args) {
		System.out.println("[MultipartMailClient]main()");
		String mail_url;
		if (args.length == 0) {
			System.err.println(
				"Usage: java MultipartMailClient protocol://username@host:port/foldername");
			//return;
			mail_url = URL;
		} else {
			mail_url = args[0];
		}
		System.out.println("[MultipartMailClient.main()]mail server : "+mail_url);
		URLName server = new URLName(mail_url);
		try {
			System.out.println("[MultipartMailClient.main()]Session erffnen");
			Session session =
				Session.getDefaultInstance(
					new Properties(),
					new MailAuthenticator(server.getUsername()));
			// Verbindungsaufbau
			System.out.println("[MultipartMailClient.main()]Folder erffnen");
			Folder folder = session.getFolder(server);
			if (folder == null) {
				System.out.println(
					"Folder " + server.getFile() + " nicht gefunden.");
				System.exit(1);
			}
			folder.open(Folder.READ_ONLY);
			System.out.println("[MultipartMailClient.main()]Messages holen");
			Message[] messages = folder.getMessages();
			for (int i = 0; i < messages.length; i++) {
				System.out.println(
					"--- Message " + (i + 1) + " ---");
				// Ausgabe der Message Headers
				Enumeration headers = messages[i].getAllHeaders();
				while (headers.hasMoreElements()) {
					Header h = (Header) headers.nextElement();
					System.out.println(h.getName() + ": " + h.getValue());
				}
				System.out.println();
				// Multiparts
				Object body = messages[i].getContent();
				if (body instanceof Multipart) {
					processMultipart((Multipart) body);
				} else { // einfache Meldung
					processPart(messages[i]);
				}
				System.out.println();
			}
			System.out.println("[MultipartMailClient.main()]Session schliessen");
			// Verbindungsabbau
			// Meldungen bleiben auf dem Server
			System.out.println("[MultipartMailClient.main()]Folder schliessen");
			folder.close(false);
		} catch (Exception e) {
			e.printStackTrace();
		}
		// GUI herunter fahren
		// exit
		System.out.println("[MultipartMailClient]main() Ende");
		System.exit(0);
	}
	public static void processMultipart(Multipart mp)
		throws MessagingException {

		for (int i = 0; i < mp.getCount(); i++) {
			processPart(mp.getBodyPart(i));
		}

	}

	public static void processPart(Part p) {

		try {
			String fileName = p.getFileName();
			String disposition = p.getDisposition();
			String contentType = p.getContentType();
			if (fileName == null
				&& (disposition.equals(Part.ATTACHMENT)
					|| !contentType.equals("text/plain"))) {
				// Zufallsnamen
				fileName = File.createTempFile("attachment", ".txt").getName();
			}
			if (fileName == null) { // inline
				p.writeTo(System.out);
			} else {
				File f = new File(fileName);
				// suchen einer Version, die noch nicht existiert
				for (int i = 1; f.exists(); i++) {
					String newName = fileName + " " + i;
					f = new File(newName);
				}
				FileOutputStream out = new FileOutputStream(f);

				// p.writeTo() funktioniert nicht
				// Wir kopieren Input auf Output Stream
				// Codierung erfolgt automatisch
				// Base-64,verschiedene Formate
				InputStream in = new BufferedInputStream(p.getInputStream());
				int b;
				while ((b = in.read()) != -1)
					out.write(b);
				out.flush();
				out.close();
				in.close();
			}
		} catch (Exception e) {
			System.err.println(e);
			e.printStackTrace();
		}

	}

}