/*
 * Created on 09.03.2004
 * SearchClient
 *
*/
package theorie;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Properties;

import javax.mail.Address;
import javax.mail.Folder;
import javax.mail.Header;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.URLName;
import javax.mail.internet.InternetAddress;
import javax.mail.search.FromTerm;
import javax.mail.search.OrTerm;
import javax.mail.search.SearchTerm;

/**
 * Durchsuchen von Foldern
 */

public class SearchClient {
	private static final String URL = "imap://jjoller@imap.hsr.ch/INBOX";
	public static void main(String[] args) {
		System.out.println("[SearchClient]main()");
		String mail_url;
		if (args.length == 0) {
			System.err.println(
				"Usage: java AllPartsClient protocol://username@host:port/foldername");
			//return;
			mail_url = URL;
		} else {
			mail_url = args[0];
		}

		System.out.println("[SearchClient]main() : mail url : "+mail_url);
		URLName server = new URLName(mail_url);

		try {
			System.out.println("[SearchClient]main() : Session erffnen");
			Session session =
				Session.getDefaultInstance(
					new Properties(),
					new MailAuthenticator(server.getUsername()));
			// Verbindungsaufbau
			System.out.println("[SearchClient]main() : Folder ffnen (Verbindungsaufbau)");
			Folder folder = session.getFolder(server);
			if (folder == null) {
				System.out.println(
					"Folder " + server.getFile() + " nicht gefunden.");
				System.exit(1);
			}
			folder.open(Folder.READ_ONLY);

			System.out.println("[SearchClient]main() : Search definieren");
			SearchTerm term = null;
			if (args.length > 1) {
				SearchTerm[] terms = new SearchTerm[args.length - 1];
				for (int i = 1; i < args.length; i++) {
					Address a = new InternetAddress(args[i]);
					terms[i - 1] = new FromTerm(new InternetAddress(args[i]));
				}
				if (terms.length > 1)
					term = new OrTerm(terms);
				else
					term = terms[0];
			}

			// Nachricht lesen
			System.out.println("[SearchClient]main() : Messages lesen");
			Message[] messages;
			if (term == null) {
				messages = folder.getMessages();
			} else {
				messages = folder.search(term);
			}
			for (int i = 0; i < messages.length; i++) {
				System.out.println(
					"------------ Message " + (i + 1) + " ------------");

				// Headers ausgeben
				Enumeration headers = messages[i].getAllHeaders();
				while (headers.hasMoreElements()) {
					Header h = (Header) headers.nextElement();
					System.out.println(h.getName() + ": " + h.getValue());
				}
				System.out.println();

				// Parts auflisten
				Object body = messages[i].getContent();
				if (body instanceof Multipart) {
					processMultipart((Multipart) body);
				} else { // Standardmeldung
					processPart(messages[i]);
				}
				System.out.println();
			}
			folder.close(false);
			System.out.println("[SearchClient]main() : Folder schliessen");
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.out.println("[SearchClient]main() : Ende");
		System.exit(0);
	}
	public static void processMultipart(Multipart mp)
		throws MessagingException {

		for (int i = 0; i < mp.getCount(); i++) {
			processPart(mp.getBodyPart(i));
		}

	}

	public static void processPart(Part p) {

		try {
			String fileName = p.getFileName();
			if (fileName == null) { // likely inline
				p.writeTo(System.out);
			} else if (fileName != null) {
				File f = new File(fileName);
				for (int i = 1; f.exists(); i++) {
					String newName = fileName + " " + i;
					f = new File(newName);
				}
				FileOutputStream out = new FileOutputStream(f);
				InputStream in = new BufferedInputStream(p.getInputStream());
				int b;
				while ((b = in.read()) != -1)
					out.write(b);
				out.flush();
				out.close();
				in.close();
			}
		} catch (Exception e) {
			System.err.println(e);
			e.printStackTrace();
		}
	}
}