package zusatzBeispiele;

/**
 * Title:        Setzen und Lesen von Message Flags
 * Description:  einfache Demo des Einsatzen von Flags
 */

import java.util.Date;
import java.util.Properties;

import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.URLName;
import javax.mail.internet.InternetAddress;

public class FlagsClient {

	public static void main(String[] args) {

		if (args.length == 0) {
			System.err.println(
				"Usage: java FlagsClient protocol://username@host:port/foldername\n"
					+ "Beispiel: pop3://fam_joller@pop.bluewin.ch/INBOX");

			System.exit(2);
		}
		System.out.println("[FlagsClient]Start");
		URLName server = new URLName(args[0]);
		System.out.println("[FlagsClient]URL : " + args[0]);

		try {

			Session session =
				Session.getDefaultInstance(
					new Properties(),
					new MailAuthenticator(server.getUsername()));

			// Verbindungsaufbau zum Server und ffnen der Folder
			System.out.println("[FlagsClient]Session  getFolder");
			Folder folder = session.getFolder(server);
			if (folder == null) {
				System.out.println(
					"[FlagsClient]Folder "
						+ server.getFile()
						+ " nicht gefunden.");
				System.exit(1);
			}

			// Flags knnte man ausgeben ber			
			Flags flags = folder.getPermanentFlags();
			System.out.println("-----------------------------------------------------");
			System.out.println("ANSWERED Flag: "+flags.contains(Flags.Flag.ANSWERED));
			System.out.println("DELETED Flag: "+flags.contains(Flags.Flag.DELETED));
			System.out.println("DRAFT Flag: "+flags.contains(Flags.Flag.DRAFT));
			System.out.println("FLAGGED Flag: "+flags.contains(Flags.Flag.FLAGGED));
			System.out.println("RECENT Flag: "+flags.contains(Flags.Flag.RECENT));
			System.out.println("SEEN Flag: "+flags.contains(Flags.Flag.SEEN));
			System.out.println("USER Flag: "+flags.contains(Flags.Flag.USER));
			System.out.println("-----------------------------------------------------");
			
			System.out.println("[FlagsClient]Session  openFolder");
			folder.open(Folder.READ_ONLY);

			// Nachrichten vom Server lesen
			Message[] messages = folder.getMessages();
			for (int i = 0; i < messages.length; i++) {
				System.out.println(
					"------------ Message " + (i + 1) + " ------------");
				// Headers bestimmen
				String from = InternetAddress.toString(messages[i].getFrom());
				if (from != null)
					System.out.println("From: " + from);
				String replyTo =
					InternetAddress.toString(messages[i].getReplyTo());
				if (replyTo != null)
					System.out.println("Reply-to: " + replyTo);
				String to =
					InternetAddress.toString(
						messages[i].getRecipients(Message.RecipientType.TO));
				if (to != null)
					System.out.println("To: " + to);
				String cc =
					InternetAddress.toString(
						messages[i].getRecipients(Message.RecipientType.CC));
				if (cc != null)
					System.out.println("Cc: " + cc);
				String bcc =
					InternetAddress.toString(
						messages[i].getRecipients(Message.RecipientType.BCC));
				if (bcc != null)
					System.out.println("Bcc: " + to);
				String subject = messages[i].getSubject();
				if (subject != null)
					System.out.println("Subject: " + subject);
				Date sent = messages[i].getSentDate();
				if (sent != null)
					System.out.println("Sent: " + sent);
				Date received = messages[i].getReceivedDate();
				if (received != null)
					System.out.println("Received: " + received);

				// Flags testen:
				if (messages[i].isSet(Flags.Flag.DELETED)) {
					System.out.println("Deleted");
				}
				if (messages[i].isSet(Flags.Flag.ANSWERED)) {
					System.out.println("Answered");
				}
				if (messages[i].isSet(Flags.Flag.DRAFT)) {
					System.out.println("Draft");
				}
				if (messages[i].isSet(Flags.Flag.FLAGGED)) {
					System.out.println("Marked");
				}
				if (messages[i].isSet(Flags.Flag.RECENT)) {
					System.out.println("Recent");
				}
				if (messages[i].isSet(Flags.Flag.SEEN)) {
					System.out.println("Read");
				}
				if (messages[i].isSet(Flags.Flag.USER)) {
					// User Flags
					// Array
					String[] userFlags = messages[i].getFlags().getUserFlags();
					for (int j = 0; j < userFlags.length; j++) {
						System.out.println("User flag: " + userFlags[j]);
					}
				}
				System.out.println();
			}
			// Verbindungsabbau
			// Nachrichten bleiben auf dem Server
			System.out.println("[FlagsClient]Folder  close");
			folder.close(false);
			System.out.println("[FlagsClient]Ende");

		} catch (Exception e) {
			e.printStackTrace();
		}
		// Abschluss der Applikation
		// exit()
		System.exit(0);
	}
}